::cisco::eem::event_register_syslog occurs 1 pattern .*%DHCP-6-ADDRESS_ASSIGN.* maxrun 40

# _________________________________________________________________________#
# |                                                                       |#
# | mac_dhcp.tcl                                                          |#
# |                                                                       |#
# | by: David Lin                                                         |#
# |  2009, Cisco Systmes, Inc.                                           |#
# |                                                                       |#
# | Version 01.0 10/12/2009                                               |#
# |_______________________________________________________________________|#
# 
# Trigger - %DHCP-6-ADDRESS_ASSIGN syslog msg
# Action  - Shutdown interface defined by envvar macaddr_wan_interface
#           if remote MAC address has changed.
#           WAN interface should be shut only if the remote P2P MAC address change
#           was due to a router reload.
#           Script should not shut interface if MAC address change was due to reload.

# Example configuration:
#
# event manger directory user policy flash:
# event manager environment macaddr_wan_interface Fa0/0
# event manager policy mac_dhcp.tcl
#
# Requirements:
#
# mac_init.tcl script - initializes envvar macaddr_old to 0000.0000.0000
#                       Example: event manager environment macaddr_old 0000.0000.0000
#      mac.tcl script - checks MAC address change of remote device when line protocol comes up
#



#------------------- Namespace imports -----------------

namespace import ::cisco::eem::*
namespace import ::cisco::lib::*


#-------------------   " cli open"   -------------------

if [catch {cli_open} result] {
  error $result $errorInfo
} else {
  array set cli $result
}

if [catch {cli_exec $cli(fd) "enable"} result] {
  error $result $errorInfo
}

#@@@@@@@@@  Proceed only if macaddr_wan_attribute == "dhcp"   @@@@@@@@@@

if { $macaddr_wan_attribute == "dhcp"} {


#-------- Fetch WAN DHCP IP Address from Syslog --------
#
#  Example:
#   
#  %DHCP-6-ADDRESS_ASSIGN: Interface GigabitEthernet0/1 assigned DHCP address 40.40.40.13, mask 255.255.255.0, 
#  hostname HUB
#

array set arr_einfo [event_reqinfo]
if {$_cerrno != 0} {
    set result [format "component=%s; subsys err=%s; posix err=%s;\n%s" \
        $_cerr_sub_num $_cerr_sub_err $_cerr_posix_err $_cerr_str]
    error $result 
}

set syslog_msg $arr_einfo(msg)

set dhcp_ipaddr_new ""
regexp {.*Interface .* assigned DHCP address ([0-9]+\.[0-9]+\.[0-9]+\.[0-9]+)} $syslog_msg match dhcp_ipaddr


#----------------------- Fetch Old MAC address from EEM envar macaddr_old ------------ 
#
#  Example(s):
#
#  HUB#sh event manager env macaddr_old
#  0000.0000.0000
#
#  HUB#sh event manager env macaddr_old
#  0016.47b2.ed40
#  

if [catch {cli_exec $cli(fd) "sh event manager env macaddr_old"} result] {
    error $result $errorInfo
  }
   set show_event_manager_env $result

regsub -all {\s+} $show_event_manager_env "" show_event_manager_env

regexp {([0-9,a-z]+\.[0-9,a-z]+\.[0-9,a-z]+)} $show_event_manager_env match macaddr_old

#action_syslog msg "Old MAC address is $macaddr_old"


#----------------------- Fetch Remote MAC address from ARP table ------------ 
#
#  The following example assumes Gig0/1 is a point-to-point link
#  and fetches the MAC address of the device on the other side of the link.  
#  40.40.40.1 is directly connected in this particular example.
#
#  Example:
#
#  HUB#sh ip arp gig0/1 | exc Address|40.40.40.1
#  Internet  40.40.40.254           34   0016.47b2.ed40  ARPA   GigabitEthernet0/1
#


# Provide 5 seconds to resolve MAC address of remote device; increase as needed
after 5000

if [catch {cli_exec $cli(fd) "sh ip arp $macaddr_wan_interface | exc Address|$dhcp_ipaddr"} result] {
    error $result $errorInfo
  }
   set show_ip_arp $result

regsub -all {\s+} $show_ip_arp " " show_ip_arp

set macaddr_new ""
regexp {Internet ([0-9]+\.[0-9]+\.[0-9]+\.[0-9]+) ([0-9]+) ([0-9,a-z]+\.[0-9,a-z]+\.[0-9,a-z]+) ARPA} $show_ip_arp match ipaddr_remote age macaddr_new

action_syslog msg "Fetching MAC address of remote peer"


#----------------------- Shut interface if macaddr_new != macaddr_old ----------------

action_syslog msg "Comparing old & new MAC addresses of remote device"

if { $macaddr_new == $macaddr_old } {
        action_syslog msg "Old & new MAC addresses are the same. No action required."

}


if { $macaddr_old == "0000.0000.0000" && $macaddr_status == "unlock" } {
        action_syslog msg "MAC address discovered during router reload."
        action_syslog msg "MAC address of remote peer will be honored and locked. "

	if [catch {cli_exec $cli(fd) "config t"} result] {
  	error $result $errorInfo
	}

	if [catch {cli_exec $cli(fd) "event manager environment macaddr_old $macaddr_new"} result] {
    	error $result $errorInfo
	}

	if [catch {cli_exec $cli(fd) "event manager environment macaddr_status lock"} result] {
    	error $result $errorInfo
  	}

	if [catch {cli_exec $cli(fd) "wr mem"} result] {
    	error $result $errorInfo
  	}

        action_syslog msg "MAC status -> locked"
}

if { $macaddr_old != $macaddr_new && $macaddr_old != "0000.0000.0000" } {

        action_syslog msg "Old & new MAC addresses not the same!"
        action_syslog msg "Interface $macaddr_wan_interface will be shut down"

	if [catch {cli_exec $cli(fd) "config t"} result] {
  	error $result $errorInfo
	}

	if [catch {cli_exec $cli(fd) "interface $macaddr_wan_interface"} result] {
    	error $result $errorInfo
	}

	if [catch {cli_exec $cli(fd) "shutdown"} result] {
    	error $result $errorInfo
	}

 }

#@@@@@@@@@@@@@@  End of if condition for macaddr_wan_attribute == "dhcp"  @@@@@@ 
} 

#--------------------- cli close ------------------------

  cli_close $cli(fd) $cli(tty_id)
