::cisco::eem::event_register_syslog occurs 1 pattern .*%LINEPROTO-5-UPDOWN.*GigabitEthernet0/1.*up maxrun 40

#---  !!!!!!! IMPORTANT !!!!!!!
#
#     Replace 'GigabitEthernet0/1' above w/ the *exact* interface name you want to track  
#
#---  !!!!!!!!!!!!!!!!!!!!!!!!!!  

# _________________________________________________________________________#
# |                                                                       |#
# | mac.tcl                                                               |#
# |                                                                       |#
# | by: David Lin                                                         |#
# |  2009, Cisco Systmes, Inc.                                           |#
# |                                                                       |#
# | Version 01.0 11/25/2009                                               |#
# |_______________________________________________________________________|#
# 
# Trigger - LINEPROTO-5-UPDOWN...<wan_interface>...up syslog msg
# Action  - Shutdown interface defined by envvar wan_interface if the remote P2P MAC address changes.
#           First MAC address learned will be placed in a "locked" state.
#           Script should not shut interface if MAC address change was due to reload.

# Example configuration:
#
# event manger directory user policy flash:
# event manager environment macaddr_wan_interface Fa0/0
# event manager policy mac.tcl
#
# Requirements:
#
# mac_init.tcl script - initializes envvar macaddr_old to 0000.0000.0000 if macaddr_status = "unlock" 
#                       


#------------------- Namespace imports -----------------

namespace import ::cisco::eem::*
namespace import ::cisco::lib::*


#-------------------   " cli open"   -------------------

if [catch {cli_open} result] {
  error $result $errorInfo
} else {
  array set cli $result
}

if [catch {cli_exec $cli(fd) "enable"} result] {
  error $result $errorInfo
}


#@@@@@@@@@@@@@  Proceed only if macaddr_wan_attribute == static @@@@@@@@@

if { $macaddr_wan_attribute == "static" } {

#--- Give time for router to learn mac address of remote device
after 5000

#-------- Fetch IP address of WAN interface --------
#
#  examples:
#   
#  HUB#sh ip int brief gig0/1 | exc Interface
#  GigabitEthernet0/1         40.40.40.1      YES NVRAM  up   <--- Static IP
#   
#  HUB#sh ip int brief gig0/1 | exc Interface
#  GigabitEthernet0/1         unassigned      YES DHCP   up   <--- DHCP   IP
#


if [catch {cli_exec $cli(fd) "sh ip int brief $macaddr_wan_interface | exc Interface"} result] {
    error $result $errorInfo
  }
   set show_ip_int_brief $result

#action_syslog msg "show ip int brief - $show_ip_int_brief"

regsub -all {\s+} $show_ip_int_brief " " show_ip_int_brief

regexp {(.*) ([0-9]+\.[0-9]+\.[0-9]+\.[0-9]+) .*} $show_ip_int_brief match interface ipaddr   

regsub -all {\s+} $interface "" interface
regsub -all {\s+} $ipaddr "" ipaddr


#action_syslog msg "WAN interface  is $interface"
#action_syslog msg "WAN IP address is $ipaddr"


#----------------------- Fetch Old MAC address from EEM envar macaddr_old ------------ 
#
#  Example(s):
#
#  HUB#sh event manager env macaddr_old
#  0000.0000.0000
#
#  HUB#sh event manager env macaddr_old
#  0016.47b2.ed40
#  

if [catch {cli_exec $cli(fd) "sh event manager env macaddr_old"} result] {
    error $result $errorInfo
  }
   set show_event_manager_env $result

regsub -all {\s+} $show_event_manager_env "" show_event_manager_env

regexp {([0-9,a-z]+\.[0-9,a-z]+\.[0-9,a-z]+)} $show_event_manager_env match macaddr_old

#action_syslog msg "Old MAC address is $macaddr_old"


#----------------------- Fetch Remote MAC address from ARP table ------------ 
#
#  The following example assumes Gig0/1 is a point-to-point link
#  and fetches the MAC address of the device on the other side of the link.  
#  40.40.40.1 is directly connected in this particular example.
#
#  Example:
#
#  HUB#sh ip arp gig0/1 | exc Address|40.40.40.1
#  Internet  40.40.40.254           34   0016.47b2.ed40  ARPA   GigabitEthernet0/1
#

if [catch {cli_exec $cli(fd) "sh ip arp $interface | exc Address|$ipaddr"} result] {
    error $result $errorInfo
  }
   set show_ip_arp $result

regsub -all {\s+} $show_ip_arp " " show_ip_arp

set macaddr_new ""
regexp {Internet ([0-9]+\.[0-9]+\.[0-9]+\.[0-9]+) ([0-9]+) ([0-9,a-z]+\.[0-9,a-z]+\.[0-9,a-z]+) ARPA} $show_ip_arp match ipaddr_remote age macaddr_new

action_syslog msg "Fetching MAC address of remote peer"


#----------------------- Shut interface if macaddr_new != macaddr_old ----------------

action_syslog msg "Comparing old & new MAC addresses of remote peer"

if { $macaddr_new == $macaddr_old } {
        action_syslog msg "Old & new MAC addresses are the same. No action required."

}

if { $macaddr_old == "0000.0000.0000" && $macaddr_status == "unlock" } {
        action_syslog msg "MAC address discovered during router reload."
        action_syslog msg "MAC address of remote peer will be honored and locked. "

	if [catch {cli_exec $cli(fd) "config t"} result] {
  	error $result $errorInfo
	}

	if [catch {cli_exec $cli(fd) "event manager environment macaddr_old $macaddr_new"} result] {
    	error $result $errorInfo
	}

	if [catch {cli_exec $cli(fd) "event manager environment macaddr_status lock"} result] {
    	error $result $errorInfo
	}

	if [catch {cli_exec $cli(fd) "wr mem"} result] {
    	error $result $errorInfo
	}
        action_syslog msg "MAC status -> locked"

}


if { $macaddr_old != $macaddr_new && $macaddr_old != "0000.0000.0000" } {

        action_syslog msg "Old & new MAC address of remote peer are not the same!"
        action_syslog msg "Shutting down interface $interface"

	if [catch {cli_exec $cli(fd) "config t"} result] {
  	error $result $errorInfo
	}

	if [catch {cli_exec $cli(fd) "interface $interface"} result] {
    	error $result $errorInfo
	}

	if [catch {cli_exec $cli(fd) "shutdown"} result] {
    	error $result $errorInfo
	}

	if [catch {cli_exec $cli(fd) "wr mem"} result] {
    	error $result $errorInfo
	}

 }


#@@@@@@@@@@@@@  End of if condition for macaddr_wan_attribute == static @@@@@@@@@
}

#--------------------- cli close ------------------------

  cli_close $cli(fd) $cli(tty_id)
